unit CalUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, ComCtrls;

type
  TCalForm = class(TForm)
    ChannelPanel: TPanel;
    Panel2: TPanel;
    WriteButton: TButton;
    WriteProgress: TProgressBar;
    Panel1: TPanel;
    Label1: TLabel;
    FileEdit: TEdit;
    Panel3: TPanel;
    Memo1: TMemo;
    comboRange: TComboBox;
    Label2: TLabel;
    buttonConfirm: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure WriteButtonClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure buttonConfirmClick(Sender: TObject);
    procedure comboRangeChange(Sender: TObject);
  private
    { Private declarations }
  public
    bGoAhead: Boolean;
    { Public declarations }
  end;

implementation

{$R *.DFM}

uses
  AIOUSB, DetectUnit;

const
  //Currently in quarter-seconds.
  EstEraseTime = 2;
  Est1To1Time  = 17;
  EstWriteTime = 54;

var
  ChannelRadios: array of TRadioGroup;
  DeviceIndex: LongWord = diOnly;

procedure TCalForm.FormCreate(Sender: TObject);
var
  I, Channels: Integer;
  Status, PID: LongWord;
  DetectForm: TDetectForm;

begin
  SetCurrentDir(ExtractFilePath(Application.ExeName));

  SetLength(ChannelRadios, 16);
  for I := 0 to High(ChannelRadios) do begin
    ChannelRadios[I] := TRadioGroup.Create(Self);
    with ChannelRadios[I] do begin
      Parent := ChannelPanel;
      Caption := 'Ch ' + IntToStr(I);
      Items.Text := 'Leave Alone'#13'Just Erase'#13'1-to-1'#13'Upload File';
      ItemIndex := 0;
    end;
  end;
  OnResize := FormResize;
  FileEdit.Align := alTop;

  Status := QueryDeviceInfo(DeviceIndex, @PID, nil, nil, nil, nil);
  if (Status <> ERROR_SUCCESS) or (PID < $8060) or (PID > $807F) then begin
    DetectForm := TDetectForm.Create(Self);
    if DetectForm.ShowModal = mrOK then begin
      DeviceIndex := DetectForm.DeviceIndex;
      DetectForm.Free;
      QueryDeviceInfo(DeviceIndex, @PID, nil, nil, nil, nil);
    end
    else begin
      DetectForm.Free;
      //Application.Terminate;
      Exit;
    end;
  end;

  case PID of
    $8070, $8071, $8078, $8079: Channels := 16;
    $8072, $8073, $807A, $807B: Channels := 12;
    $8074, $8075, $807C, $807D: Channels := 8;
    $8076, $8077, $807E, $807F: Channels := 4;
    else Exit;
  end;
  for I := Channels to High(ChannelRadios) do ChannelRadios[I].Visible := False;

  bGoAhead := True;
end;

procedure TCalForm.FormResize(Sender: TObject);
var
  XEdge, YEdge: array of Integer;
  X, Y: Integer;

begin
  LockWindowUpdate(Handle);
  SetLength(XEdge, 5);
  SetLength(YEdge, 5);
  XEdge[4] := ChannelPanel.Width;
  YEdge[4] := ChannelPanel.Height;
  for X := 0 to 3 do
    XEdge[X] := Round(X / 4 * XEdge[4])
  ;
  for Y := 0 to 3 do
    YEdge[Y] := Round(Y / 4 * YEdge[4])
  ;
  for Y := 0 to 3 do
    for X := 0 to 3 do
      ChannelRadios[Y * 4 + X].SetBounds(XEdge[X], YEdge[Y], XEdge[X + 1] - XEdge[X], YEdge[Y + 1] - YEdge[Y])
  ;
  LockWindowUpdate(0);
end;

procedure TCalForm.WriteButtonClick(Sender: TObject);
const
  IndexTime: array[0..3] of Integer = (0, EstEraseTime, EstEraseTime + Est1To1Time, EstEraseTime + EstWriteTime);

var
  A, I, L: Integer;
  Buf: AnsiString;
  Status: LongWord;
  one: AnsiString;

begin
  L := 0;
  for I := 0 to High(ChannelRadios) do
    L := L + IndexTime[ChannelRadios[I].ItemIndex]
  ;
  WriteProgress.Max := L;
  WriteProgress.Visible := True;

  L := 0;
  for I := 0 to High(ChannelRadios) do begin
    A := ChannelRadios[I].ItemIndex;

    Status := ERROR_SUCCESS; //It's for the compiler.
    case A of
      1: begin
        Status := AIOUSB_FlashEraseSector(DeviceIndex, I * 2);
        if Status = ERROR_SUCCESS then
          Status := AIOUSB_FlashEraseSector(DeviceIndex, I * 2 + 1)
        ;
      end;
      2: begin
        one := ':1TO1:';
        Status := DACSetChannelCal(DeviceIndex, I, PAnsiChar(one));
      end;
      3: begin
        Buf := FileEdit.Text;
        Status := DACSetChannelCal(DeviceIndex, I, PAnsiChar(Buf));
        Buf := '';
      end;
      else Continue;
    end;
    ChannelRadios[I].ItemIndex := 0;

    if Status <> ERROR_SUCCESS then
      ChannelRadios[I].Caption := 'Error ' + IntToStr(Status)
    ;

    L := L + IndexTime[A];
    WriteProgress.Position := L;
    Application.ProcessMessages;
    Sleep(1000);
    Application.ProcessMessages;
  end;

  WriteProgress.Visible := False;
  WriteProgress.Position := 0;
  Beep;
end;

procedure TCalForm.buttonConfirmClick(Sender: TObject);
begin
    if DACSetBoardRange(DeviceIndex, comboRange.ItemIndex) = 0 then begin
        buttonConfirm.caption := 'Confirmed';
        buttonConfirm.enabled := false;
        ChannelPanel.Visible := true;
      end;
end;

procedure TCalForm.comboRangeChange(Sender: TObject);
begin
    if comboRange.itemindex = -1 then
        buttonConfirm.enabled := false
    else begin
        buttonConfirm.enabled := true;
        buttonConfirm.Caption := 'Confirm Selection';
    end;
end;

procedure TCalForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Action := caFree;
end;

end.
